--------------------------------------------------------------------------------
-- Universidad Autonoma de Madrid
-- Escuela Politecnica Superior
-- Laboratorio de DIE
--------------------------------------------------------------------------------
-- Block: pico_example_tb
--
-- Author: (UAM)
--
-- Description:
--   Simple testbench for pico_example design (a simple demo design running
--   on Spartan-3 Starter Kit Board, S3BOARD, implementing a stopwatch with
--   an embedded PicoBlaze processor).
--   This is not an automatic testbench with good functional coverage,
--   the intended usage is only to be able to start a simulation and
--   to see some waveforms, and to take it as a starting point for a more
--   complete testbench. 
--   The testbench starts the stopwatch and lets some time run.
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------------------------------
-- Entity declaration
--------------------------------------------------------------------------------

entity pico_example_tb is
  
end pico_example_tb;

--------------------------------------------------------------------------------
-- Architecture
--------------------------------------------------------------------------------

architecture tb of pico_example_tb is

  -- Component declarations
  -------------------------
  
  -- Unit under test: PicoBlaze design example (stopwatch):
  component pico_example
    port (
          -- System clock input from board:
          CLK    : in  std_logic;
          -- Inputs from board: switches and push buttons:
          SW     : in  std_logic_vector (7 downto 0);
          BTN    : in  std_logic_vector (3 downto 0);
          -- Outputs to board: LEDs and 4x7-segment display:
          LD     : out std_logic_vector (7 downto 0);
          SEG_A  : out std_logic;
          SEG_B  : out std_logic;
          SEG_C  : out std_logic;
          SEG_D  : out std_logic;
          SEG_E  : out std_logic;
          SEG_F  : out std_logic;
          SEG_G  : out std_logic;
          SEG_DP : out std_logic;
          AN     : out std_logic_vector (3 downto 0)
         );
  end component;

  -- Constants:
  -------------
  
  constant PERIOD : time := 10 ns;  -- Clock period

  -- Signals:
  -----------

  -- Signals (real hardware):
  signal reset : std_logic;  -- System reset
  signal clk   : std_logic;  -- System clock
  signal sw    : std_logic_vector (7 downto 0); -- Slide switches
  signal btn   : std_logic_vector (3 downto 0); -- Pushbuttons
  signal ld    : std_logic_vector (7 downto 0); -- LEDs
  signal seg   : std_logic_vector (7 downto 0); -- 7-seg segments (cathodes)
  signal an    : std_logic_vector (3 downto 0); -- 7-seg digit activation(anode)

  -- Signals (testbench internals):
  signal end_sim   : boolean := false; -- End of simulation flag
  signal sig_button_zero     : std_logic := '0'; -- Toggle to emulate pushing
                                                 --   count reset button
  signal sig_button_strtstop : std_logic := '0'; -- Toggle to emulate pushing
                                                 --   start/stop button

  -- Procedures:
  --------------

  -- Wait for a number of cycles:
  procedure wait_clock_cycles (ncyc : in integer) is
  begin
    for idx in 1 to ncyc loop
      wait until falling_edge(clk);
    end loop;
  end wait_clock_cycles;
        
  -- Push a button:
  procedure push_a_button (signal sig_button : inout std_logic) is
  begin
    sig_button <= not sig_button;
  end push_a_button;

  
begin  -- tb

  -- Button input connections: We don't use BTN[0] and we fix it to 0,
  --   while BTN[3] is the reset button, so connect it to testbench reset
  --   signal:  
  btn (0) <= '0';
  btn (3) <= reset;

  -- Component instantiation: Unit Under Test:
  i_pico_example : pico_example
    port map (
              -- System clock input from board:
              CLK    => clk,
              -- Inputs from board: switches and push buttons:
              SW     => sw (7 downto 0),
              BTN    => btn (3 downto 0),
              -- Outputs to board: LEDs and 4x7-segment display:
              LD     => ld (7 downto 0),
              SEG_A  => seg (0),
              SEG_B  => seg (1),
              SEG_C  => seg (2),
              SEG_D  => seg (3),
              SEG_E  => seg (4),
              SEG_F  => seg (5),
              SEG_G  => seg (6),
              SEG_DP => seg (7),
              AN     => an (3 downto 0)
             );

  -- Clock generation:
  clock_gen : process
  begin
    while not end_sim loop
      clk <= '0';
      wait for PERIOD/2;
      clk <= '1';
      wait for PERIOD/2;
    end loop;
    wait;
  end process clock_gen;
  
  -- Button for resetting count: Generate a high-level pulse:
  process
  begin
    btn(2) <= '0';
    wait on sig_button_zero;
    btn(2) <= '1';
    wait for 100*PERIOD;
  end process;
  
  -- Button for start/stop: Generate a high-level pulse:
  process
  begin
    btn(1) <= '0';
    wait on sig_button_strtstop;
    btn(1) <= '1';
    wait for 100*PERIOD;
  end process;
  
    
  -- Main process:

  process
  begin
    report "SIMULATION START";
    
    -- Initialize signals, with initial reset:
    reset <= '1';
    sw    <= "00000000";

    -- Wait some cycles and de-assert reset:
    wait_clock_cycles (10);
    reset <= '0';

    -- Wait some cycles and push the start/stop button:
    wait_clock_cycles (10);
    push_a_button (sig_button_strtstop);

    -- Wait for some time to see some nice waveforms...:
    wait_clock_cycles (10000);

    report "SIMULATION END";
    end_sim <= true;
    wait;
  end process;
    
end tb;
